local t = require('luatest')
local g = t.group('uroboros_replicator')
local helper = require('test.helper.integration')
local requests = require('test.requests')

g.test_ext_lib = function()
    local replicator = require('uroboros-cartridge-sidecar.roles.uroboros-replicator')
    local instance = helper.get_master_by_role_name(helper.cluster, replicator.role_name)
    local res = instance.net_box:call('liburoboros_test_lib')
    t.assert(res)
end

local function get_storage_master_slave()
    local cluster = helper.cluster;

    local replicators = helper.get_servers_by_role_name(cluster, require('uroboros-cartridge-sidecar.roles.uroboros-replicator').role_name)

    local replicaset_uuid
    local master
    local slave

    for _, server in ipairs(replicators) do
        if not replicaset_uuid then
            replicaset_uuid = server.replicaset_uuid
        end
        if replicaset_uuid == server.replicaset_uuid then
            local readonly = helper.is_server_readonly(server)
            if readonly and slave == nil then
                slave = server
            end

            if not readonly and master == nil then
                master = server
            end
        end
        if master ~= nil and slave ~= nil then break end
    end

    return master, slave
end

g.test_metrics = function()
    local master, slave = get_storage_master_slave()
    t.assert_not_equals(master.http_port, slave.http_port)

    -- master metrics should have alias and uuid
    local master_requests = requests.new({ port = master.http_port, prefix = '/' })
    local master_metrics = master_requests:metrics().body
    t.assert_str_contains(master_metrics, "replicaset_alias=")
    t.assert_str_contains(master_metrics, "replicaset_uuid=")

    -- slave metrics should not have alias and uuid
    local slave_requests = requests.new({ port = slave.http_port, prefix = '/' })
    local slave_metrics = slave_requests:metrics().body
    t.assert_not_str_contains(slave_metrics, "replicaset_alias=")
    t.assert_not_str_contains(slave_metrics, "replicaset_uuid=")

    -- making master non-electable, so slave becomes a master
    master:graphql({
        query = [[
            mutation editTopology($replicasets: [EditReplicasetInput!], $servers: [EditServerInput!]) {
                cluster {
                    edit_topology(replicasets: $replicasets, servers: $servers) {
                        servers {
                            uuid
                        }
                    }
                }
            }
        ]],
        variables = {
            servers = {{
                uuid = master.instance_uuid,
                electable = false
            }}
        }
    })

    -- confirm that failover happened
    t.assert_eval_to_true(helper.is_server_readonly(master))
    t.assert_eval_to_false(helper.is_server_readonly(slave))

    -- master metrics now should NOT have alias and uuid
    local master_requests = requests.new({ port = master.http_port, prefix = '/' })
    local master_metrics = master_requests:metrics().body
    t.assert_not_str_contains(master_metrics, "replicaset_alias=")
    t.assert_not_str_contains(master_metrics, "replicaset_uuid=")

    -- slave metrics now should have alias and uuid
    local slave_requests = requests.new({ port = slave.http_port, prefix = '/' })
    local slave_metrics = slave_requests:metrics().body
    t.assert_str_contains(slave_metrics, "replicaset_alias=")
    t.assert_str_contains(slave_metrics, "replicaset_uuid=")
end